#! /usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author  : MG
@Time    : 2016/12/12 14:47
@File    : mess.py
@contact : mmmaaaggg@163.com
@desc    :
"""
DeprecationWarning(f"ibats_utils.mess 模块即将被废弃,请根据需要引用对应模块的包: "
                   f"ibats_utils.iter, ibats_utils.json, ibats_utils.math, ibats_utils.os, ibats_utils.pandas, "
                   f"ibats_utils.path, ibats_utils.decorators, ibats_utils.transfer, etc.")
# noinspection PyUnresolvedReferences
from ibats_utils.decorators import *
# noinspection PyUnresolvedReferences
from ibats_utils.iter import *
# noinspection PyUnresolvedReferences
from ibats_utils.json import *
# noinspection PyUnresolvedReferences
from ibats_utils.math import *
# noinspection PyUnresolvedReferences
from ibats_utils.os import *
# noinspection PyUnresolvedReferences
from ibats_utils.pandas import *
# noinspection PyUnresolvedReferences
from ibats_utils.path import *
# noinspection PyUnresolvedReferences
from ibats_utils.transfer import *
